#!/etc/opt/openit/perl -w
#
# Replace/update the httpd.conf script to have a support on 8880 on non https 
#
#

    use strict;
    use Cwd;

    my $config_dir = &get_directive_from_config("CONFIG_DIR");
    my $httpd_config_file = "$config_dir/httpd/httpd.conf";
    my $httpd_ssl_marker = "$config_dir/httpd/use_ssl";
    my $http_port;
    my $server_name = &get_directive_from_config( "SERVER" );
    my $httpd_user = &get_directive_from_config( "SERVER_UID_NAME" );
    my $httpd_group = &get_directive_from_config( "SERVER_GID_NAME" );
    my $log_dir = &get_directive_from_config("ROOT_TEMP_DIR") . "/log";
    my $temp_dir = &get_directive_from_config("TEMP_DIR");
    my $httpd_dir= &get_directive_from_config("HTTPD_DIR");
    my $http_loopback;
    my $loopback="127\.0\.0\.1";
    my $new_httpd_conf_file = getcwd() . "/files/httpd.conf";
    my $patch_embeded_page = getcwd() . "/files/embedded_index.pl";
    #backup current file
    system("cp -f $httpd_config_file $httpd_config_file.old");
    system("cp -f $httpd_dir/embedded_index.pl $httpd_dir/embedded_index.old");
    system("cp -f $patch_embeded_page $httpd_dir");
    

    #determine if the ssl marker is present to proceed to port specific changes
    if (-e $httpd_ssl_marker ) {
        $http_port = &get_http_port("_default_");
        $http_loopback = &get_http_port("$loopback");
        
        my $success = system("cp -f $new_httpd_conf_file $httpd_config_file ");
            
            if ($http_port ne "80"){
                &update_httpd_config_file($http_loopback,$http_port);
            }
            else{
                print "Updating httpd config...SUCCSESS";
            }
        }
    else{
        $http_port = &get_http_port("Port");
        $http_loopback = &get_http_port("$loopback");
        
        my $success = system("cp -f $new_httpd_conf_file $httpd_config_file ");
        
            if ($http_port ne "443"){
                &update_httpd_config_file($http_loopback,$http_port);
            }
            else{
                print "Updating httpd config...SUCCSESS";
            }
            
        }


        
 ###############################################################################
#
# NAME:
#   update_httpd_config_file ( modified )
# PURPOSE:
#   Update the httpd.conf with the newest config then applies the previous 
#   configured port  
# ARGS:
#   httpd_type and httpd_port
# RETURNS:
#   None
#   


sub update_httpd_config_file {
    my ( $loopback_port,$httpd_port ) = @_;

    #my $config_dir = &get_directive_from_config("CONFIG_DIR");
    #my $httpd_config_file = "$config_dir/httpd/httpd.conf";
    my $http_port_set = "0";
    if (open(IFILE, $httpd_config_file)) {
        if (open(OFILE, ">$httpd_config_file.$$")) {
            print "Updating httpd config...";
            while (<IFILE>) {
                if ($_ =~ m/^User /) {
                    $_ = "User $httpd_user\n";
                } elsif ($_ =~ m/^Group /) {
                    $_ = "Group $httpd_group\n";
                } elsif ($_ =~ m/^Port /) {
                    $_ = "Port $httpd_port\n";
                } elsif ($_ =~ m/^\#?ServerName /) {
                    $_ = "ServerName $server_name\n";
                } elsif ($_ =~ m/^\#?PidFile /) {
                    $_ = "PidFile $temp_dir/httpd/httpd.pid\n";
                } elsif ($_ =~ m|^(.*) /opt/openit/logs/(.*)$|) {
                    $_ = "$1 $log_dir/httpd/$2\n";
                } elsif ($_ =~ m|^(.*) /opt/openit/conf/(.*)$|) {
                    $_ = "$1 $config_dir/httpd/$2\n";
                }

                if (-e $httpd_ssl_marker )  { 
                    # Replace with the new HTTPS port
                    #Replace First Listen instance of http
                    if ($_ =~ m/^Listen / && $http_port_set == "0") {
                    $_ = "";
                    $http_port_set = "1";
                    } 
                    $_ =~ s/443/$httpd_port/;
                    $_ =~ s/$loopback:8880/$loopback:$loopback_port/;
                }
                else
                {
                    #Replace First Listen instance of http
                    if ($_ =~ m/^Listen / && $http_port_set == "0") {
                    $_ = "Listen $httpd_port\n";
                    $http_port_set = "1";
                    } 
                    $_ =~ s/$loopback:8880/$loopback:$loopback_port/;
                }
                
                print OFILE $_;
            }
            close OFILE;
            rename("$httpd_config_file.$$", $httpd_config_file)
              || die "Failed to rename httpd's configuration file: $!";
            
        }
        close IFILE;
        print "SUCCESS\n";
    }

}

    
        
###############################################################################
#
# NAME:
#   get_directive_from_config
# PURPOSE:
#   Get directive from host specific configuration file.
#   If directive holds another directive, call recursively.
# ARGS:
#   Directive name
# RETURNS:
#   String holding directive, or undefined if not found
#

sub get_directive_from_config {
    my ( $directive_name ) = @_;

    open( INFILE, "< /etc/opt/openit/openit.cfg " );
    my $line;
    while( defined($line = <INFILE>) && $line !~ m/^$directive_name\s+/ ) {}
    close( INFILE );
    if ( ! defined($line)) {
        # Directive not found
        return( undef );
    }
    chomp($line);

    my ( $dummy, $value ) = split(/\s+/, $line);

    if ( $value =~ m!^\$([A-Z_]+)/! ) {
        my $sub_value = &get_directive_from_config( $1 );
        $value =~ s!^(\$[A-Z_]+)/!$sub_value/!;
    }

    return( $value );
}

###############################################################################
#
# NAME:
#   get_http_port
# PURPOSE:
#   Get directive the port from http file.
#   
# ARGS:
#   port type
# RETURNS:
#   String holding directive
#

sub get_http_port {
    my ($string_key) = @_;
    
    open( INFILE1, "< $httpd_config_file " );
       
    my $line;
    while( defined($line = <INFILE1>) && $line !~ m/$string_key/ ) {}
    close( INFILE1 );
    if ( ! defined($line)) {
        # Directive not found
        return( undef );
    }
    if ($string_key eq "127.0.0.1"){
                
      my ($dummy, $value) = split(/\:/,$line);
      $line = $value;
    }
    else
    {
        $line =~ s/\D//g;
    }
   return($line);
}
